"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const arg_1 = __importDefault(require("arg"));
const interactive_1 = require("./interactive");
const execute_1 = require("./execute");
process.on('unhandledRejection', (error) => {
    throw error;
});
const args = arg_1.default({
    // Types
    '--version': Boolean,
    '--help': Boolean,
    '--verbose': Boolean,
    '--dry': Boolean,
    '--interactive': Boolean,
    // Aliases
    '-v': '--version',
    '-h': '--help',
    '-d': '--dry',
    '-i': '--interactive',
}, {
    permissive: false,
});
if (args['--version']) {
    console.log(require('../package.json').version);
    process.exit(0);
}
if (args['--help']) {
    console.log(`
    Usage
      > relabel [pattern] [old] [new]

    Interactive Mode
      > relable

    Modify
      > relable '**/*.spec.js' spec test  => dir/file.test.js

    Remove
      > relable '**/foo-bar.js' foo-  =>  dir/bar.js

    Options
      --version, -v       Version number
      --help, -h          Displays this message
      --dry, -d           Dry-run mode, does not modify files
      --interactive, -i   Interactive Mode, automatically used when not all argument are provided
`);
    process.exit(0);
}
const [pattern, search, replace = ''] = args._.slice(0, 3);
if (args['--interactive'] || !pattern || !search) {
    interactive_1.interactive({
        initialPattern: pattern,
        initialSearch: search,
        initialReplace: replace,
    })
        .then((result) => {
        execute_1.execute({ ...result, dry: !!args['--dry'] });
    })
        .catch((error) => {
        if (error === 'abort') {
            process.exit(1);
        }
        throw error;
    });
}
else {
    execute_1.execute({ pattern, search, replace, dry: !!args['--dry'] });
}
