"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.UncontrolledTextInput = exports.Input = void 0;
const jsx_runtime_1 = require("react/jsx-runtime");
const react_1 = require("react");
const ink_1 = require("ink");
const chalk_1 = __importDefault(require("chalk"));
const TextInput = ({ value: originalValue, placeholder = '', focus = true, mask, highlightPastedText = false, showCursor = true, onChange, onSubmit, }) => {
    const [{ cursorOffset, cursorWidth }, setState] = react_1.useState({
        cursorOffset: (originalValue || '').length,
        cursorWidth: 0,
    });
    react_1.useEffect(() => {
        setState((previousState) => {
            if (!focus || !showCursor) {
                return previousState;
            }
            const newValue = originalValue || '';
            if (previousState.cursorOffset > newValue.length - 1) {
                return {
                    cursorOffset: newValue.length,
                    cursorWidth: 0,
                };
            }
            return previousState;
        });
    }, [originalValue, focus, showCursor]);
    const cursorActualWidth = highlightPastedText ? cursorWidth : 0;
    const value = mask ? mask.repeat(originalValue.length) : originalValue;
    let renderedValue = value;
    let renderedPlaceholder = placeholder ? chalk_1.default.grey(placeholder) : undefined;
    // Fake mouse cursor, because it's too inconvenient to deal with actual cursor and ansi escapes
    if (showCursor && focus) {
        renderedPlaceholder =
            placeholder.length > 0
                ? chalk_1.default.inverse(placeholder[0]) + chalk_1.default.grey(placeholder.slice(1))
                : chalk_1.default.inverse(' ');
        renderedValue = value.length > 0 ? '' : chalk_1.default.inverse(' ');
        let i = 0;
        for (const char of value) {
            if (i >= cursorOffset - cursorActualWidth && i <= cursorOffset) {
                renderedValue += chalk_1.default.inverse(char);
            }
            else {
                renderedValue += char;
            }
            i++;
        }
        if (value.length > 0 && cursorOffset === value.length) {
            renderedValue += chalk_1.default.inverse(' ');
        }
    }
    ink_1.useInput((input, key) => {
        if (key.upArrow ||
            key.downArrow ||
            (key.ctrl && input === 'c') ||
            key.tab ||
            (key.shift && key.tab)) {
            return;
        }
        if (key.return) {
            if (onSubmit) {
                onSubmit(originalValue);
            }
            return;
        }
        let nextCursorOffset = cursorOffset;
        let nextValue = originalValue;
        let nextCursorWidth = 0;
        if (key.leftArrow) {
            if (showCursor) {
                nextCursorOffset--;
            }
        }
        else if (key.rightArrow) {
            if (showCursor) {
                nextCursorOffset++;
            }
        }
        else if (key.backspace || key.delete) {
            if (cursorOffset > 0) {
                nextValue =
                    originalValue.slice(0, cursorOffset - 1) +
                        originalValue.slice(cursorOffset, originalValue.length);
                nextCursorOffset--;
            }
        }
        else {
            nextValue =
                originalValue.slice(0, cursorOffset) +
                    input +
                    originalValue.slice(cursorOffset, originalValue.length);
            nextCursorOffset += input.length;
            if (input.length > 1) {
                nextCursorWidth = input.length;
            }
        }
        if (cursorOffset < 0) {
            nextCursorOffset = 0;
        }
        if (cursorOffset > originalValue.length) {
            nextCursorOffset = originalValue.length;
        }
        setState({
            cursorOffset: nextCursorOffset,
            cursorWidth: nextCursorWidth,
        });
        if (nextValue !== originalValue) {
            onChange(nextValue);
        }
    }, { isActive: focus });
    return (jsx_runtime_1.jsx(ink_1.Text, { children: placeholder
            ? value.length > 0
                ? renderedValue
                : renderedPlaceholder
            : renderedValue }, void 0));
};
exports.Input = TextInput;
const UncontrolledTextInput = (props) => {
    const [value, setValue] = react_1.useState('');
    return jsx_runtime_1.jsx(TextInput, Object.assign({}, props, { value: value, onChange: setValue }), void 0);
};
exports.UncontrolledTextInput = UncontrolledTextInput;
