"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.interactive = void 0;
const jsx_runtime_1 = require("react/jsx-runtime");
const react_1 = require("react");
const ink_1 = require("ink");
const getFiles_1 = require("./getFiles");
const interactive = async ({ initialPattern, initialSearch, initialReplace, }) => {
    return new Promise((resolve, reject) => {
        let clear;
        const App = () => {
            const { exit } = ink_1.useApp();
            const [pattern, setPattern] = react_1.useState(initialPattern || '');
            const [search, setSearch] = react_1.useState(initialSearch || '');
            const [replace, setReplace] = react_1.useState(initialReplace || '');
            const [cursur, setCursor] = react_1.useState({ cursorOffset: 0, activeInput: 0 });
            const { activeInput, cursorOffset } = cursur;
            const [files, setFiles] = react_1.useState([]);
            const setCursorOffset = (offset) => setCursor({ cursorOffset: offset, activeInput });
            const getInput = (index) => {
                if (index === 0) {
                    return {
                        value: pattern,
                        setValue: (p) => {
                            if (!p) {
                                setFiles([]);
                            }
                            else {
                                getFiles_1.getFiles(p).then((f) => {
                                    setFiles(f);
                                });
                            }
                            setPattern(p);
                        },
                    };
                }
                if (index === 1) {
                    return { value: search, setValue: setSearch };
                }
                if (index === 2) {
                    return { value: replace, setValue: setReplace };
                }
                throw new Error(`activeInput ${activeInput} is invalid`);
            };
            ink_1.useInput((input, key) => {
                const { value, setValue } = getInput(activeInput);
                if (key.escape || (key.ctrl && (input === 'c' || input === 'd'))) {
                    clear();
                    exit();
                    reject('abort');
                    return;
                }
                else if (key.ctrl && input === 'u') {
                    setValue('');
                    setCursorOffset(0);
                }
                else if (key.ctrl && input === 'a') {
                    setCursorOffset(0);
                }
                else if (key.ctrl && input === 'e') {
                    setCursorOffset(value.length);
                }
                else if (key.leftArrow) {
                    if (cursorOffset > 0) {
                        setCursorOffset(cursorOffset - 1);
                    }
                }
                else if (key.rightArrow) {
                    if (cursorOffset < value.length) {
                        setCursorOffset(cursorOffset + 1);
                    }
                }
                else if (key.return) {
                    clear();
                    exit();
                    resolve({ pattern, search, replace });
                    return;
                }
                else if (key.upArrow || (key.tab && key.shift)) {
                    const newInput = (activeInput + 3 - 1) % 3;
                    setCursor({
                        activeInput: newInput,
                        cursorOffset: key.tab
                            ? 0
                            : Math.min(cursorOffset, getInput(newInput).value.length),
                    });
                }
                else if (key.downArrow || key.tab) {
                    const newInput = (activeInput + 3 + 1) % 3;
                    setCursor({
                        activeInput: newInput,
                        cursorOffset: key.tab
                            ? 0
                            : Math.min(cursorOffset, getInput(newInput).value.length),
                    });
                }
                else {
                    if (key.delete || key.backspace) {
                        if (cursorOffset > 0) {
                            setCursorOffset(cursorOffset - 1);
                            setValue(value.slice(0, cursorOffset - 1) + value.slice(cursorOffset));
                        }
                    }
                    else {
                        setValue(value.slice(0, cursorOffset) + input + value.slice(cursorOffset));
                        setCursorOffset(cursorOffset + input.length);
                    }
                }
            });
            let searchRegexp;
            let searchError = '';
            try {
                searchRegexp = new RegExp(search);
            }
            catch (error) {
                searchError = error.message;
            }
            return (jsx_runtime_1.jsxs(jsx_runtime_1.Fragment, { children: [jsx_runtime_1.jsxs(ink_1.Box, Object.assign({ flexDirection: "column" }, { children: [jsx_runtime_1.jsx(Input, { placeholder: "Pattern", active: activeInput === 0, value: pattern, cursorOffset: cursorOffset }, void 0),
                            jsx_runtime_1.jsx(Input, { placeholder: "Search ", active: activeInput === 1, value: search, errorMessage: searchError, cursorOffset: cursorOffset }, void 0),
                            jsx_runtime_1.jsx(Input, { placeholder: "Replace", active: activeInput === 2, value: replace, cursorOffset: cursorOffset }, void 0)] }), void 0),
                    jsx_runtime_1.jsx(Monitor, { files: files, searchRegexp: searchRegexp, replace: replace }, void 0)] }, void 0));
        };
        const Input = ({ placeholder, active, value, cursorOffset, errorMessage, }) => {
            const textWithCursor = (text) => {
                if (cursorOffset === text.length) {
                    return (jsx_runtime_1.jsxs(ink_1.Text, { children: [text, jsx_runtime_1.jsx(ink_1.Text, Object.assign({ inverse: true }, { children: " " }), void 0)] }, void 0));
                }
                return (jsx_runtime_1.jsxs(jsx_runtime_1.Fragment, { children: [jsx_runtime_1.jsx(ink_1.Text, { children: text.substr(0, cursorOffset) }, void 0),
                        jsx_runtime_1.jsx(ink_1.Text, Object.assign({ inverse: true }, { children: text.substr(cursorOffset, 1) }), void 0),
                        jsx_runtime_1.jsx(ink_1.Text, { children: text.substr(cursorOffset + 1) }, void 0)] }, void 0));
            };
            return (jsx_runtime_1.jsxs(ink_1.Text, { children: [jsx_runtime_1.jsx(ink_1.Text, Object.assign({ color: active ? 'magenta' : 'white' }, { children: '| ' }), void 0),
                    !value ? (jsx_runtime_1.jsx(ink_1.Text, Object.assign({ color: "dim" }, { children: active ? textWithCursor(placeholder) : placeholder }), void 0)) : (jsx_runtime_1.jsxs(ink_1.Text, { children: [active ? textWithCursor(value) : value, jsx_runtime_1.jsxs(ink_1.Text, Object.assign({ color: "red" }, { children: [" ", errorMessage] }), void 0)] }, void 0))] }, void 0));
        };
        const Monitor = ({ files, searchRegexp, replace, }) => {
            if (files.length === 0) {
                return jsx_runtime_1.jsx(jsx_runtime_1.Fragment, {}, void 0);
            }
            return (jsx_runtime_1.jsx(jsx_runtime_1.Fragment, { children: files
                    .map((filePath) => {
                    if (!searchRegexp) {
                        return jsx_runtime_1.jsx(ink_1.Text, { children: filePath }, filePath);
                    }
                    let result = searchRegexp.exec(filePath);
                    if (result === null) {
                        return jsx_runtime_1.jsx(ink_1.Text, { children: filePath }, filePath);
                    }
                    const match = result[0];
                    const matchStartIndex = result.index;
                    return (jsx_runtime_1.jsxs(ink_1.Text, { children: [jsx_runtime_1.jsx(ink_1.Text, { children: filePath.substr(0, matchStartIndex) }, void 0),
                            jsx_runtime_1.jsx(ink_1.Text, Object.assign({ strikethrough: true, color: "red" }, { children: match }), void 0),
                            jsx_runtime_1.jsx(ink_1.Text, Object.assign({ color: "green" }, { children: replace }), void 0),
                            jsx_runtime_1.jsx(ink_1.Text, { children: filePath.substr(matchStartIndex + match.length) }, void 0)] }, filePath));
                })
                    .slice(0, process.stdout.rows - 5) }, void 0));
        };
        const inkRenderApi = ink_1.render(jsx_runtime_1.jsx(App, {}, void 0), { exitOnCtrlC: false });
        clear = inkRenderApi.clear;
    });
};
exports.interactive = interactive;
